# Copyright (C) 2003, 2004 Gregoire Lejeune <gregoire.lejeune@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

module Asp
  class Request
    require 'asp/params'
    require 'asp/common'

    public
	# Returns the value of the input of name used in a form with POST method.
    def form( key )
      if @oApacheRequest.request_method == "POST" 
        if @params[key].nil? == false and @params[key].size > 0
          if @params[key].size == 1
            return @params[key][0]
          else
            return @params[key]
          end
        else
          return nil
        end
      end
    end
  
    # Returns the value of the input of name used in a form with GET method.
    def queryString( key )
      if @oApacheRequest.request_method == "GET" or
         @oApacheRequest.request_method == "HEAD"
        if @params[key].nil? == false and @params[key].size > 0
          if @params[key].size == 1
            return @params[key][0]
          else
            return @params[key]
          end
        else
          return nil
        end
      end
    end

    # API extension. Returns the unparsed query string.
	def query
	  @param.getQuery( )
	end

    # Returns the value of the Cookie with name. If a key is specified, then a 
	# lookup will be done on the cookie as if it were a query string.
    def cookies( name=nil, key=nil )
      if name.nil? == true
        return @cookies
      elsif key.nil? == true and @cookies[name].nil? == false
        return @cookies[name]
      elsif String( @cookies[name].class ) == "Hash" and @cookies[name][key].nil? == false
        return @cookies[name][key]
      end

      return nil
    end

    # API extension. Returns the client HTTP request method, as in GET or POST.
    def method
      return @oApacheRequest.request_method
    end

    # The amount of data sent by the client in the body of the request, usually 
	# the length of the form data. This is the same value as
	#     $Request.serverVariables('CONTENT_LENGTH')
    def totalBytes
      return @oApacheRequest.headers_in["Content-Length"]
    end

    # Returns the value of the server variable / environment variable with name name. 
	# If name is not specified, returns a ref to a hash of all the server / environment 
	# variables data. The following would be a common use of this method:
    # 
	#     env = $Request.serverVariables();
	#     # env here would be equivalent to the ENV.
    def serverVariables( name=nil )
      if name.nil? == true
        return ENV.untaint
      else
        return ENV[name].untaint
      end
    end

    # To be added
    def acceptTypes 
	  ## -> string[]
	  return @oApacheRequest.headers_in["Accept"].split( %r{,\s*} )
	end

    # TODO
	def applicationPath 
	  ## -> string
	end

    # TODO
	def browser
	  ## -> System.Web.HttpBrowserCapabilities
	end

    # TODO
	def clientCertificate
	  ## -> System.Web.HttpClientCertificate
	end

    # TODO
	def contentEncoding
	  ## -> System.Text.Encoding
	end

    alias :contentLength :totalBytes
	
    # TODO
	def contentType
	  ## -> string
	end
	
    # To be added
	def currentExecutionFilePath
	  ## -> string
	  return serverVariables( 'REQUEST_URI' )
	end

    alias :filePath :currentExecutionFilePath

    # TODO
	def files
	  ## -> System.Web.HttpFileCollection
	end
	
    # TODO
	def filter
	  ## -> stream
	end

    # To be added
	def headers
	  ## -> System.Collections.Specialized.NameValueCollection
	  return @oApacheRequest.headers_in
	end

    alias :httpMethod :method

    # TODO
    def inputStream
	  ## -> stream
	end
	
    # TODO
	def isAuthenticated
	  ## -> bool
	end

    # TODO
	def isSecureConnection
	  ## -> bool
	end

    # TODO
	def item( xKey )
	  ## -> string
	end

    # TODO
	def params
	  ## -> System.Collections.Specialized.NameValueCollection
	end

    alias :path :currentExecutionFilePath

    # TODO
	def pathInfo
	  ## -> string
	end

    # TODO
	def physicalApplicationPath
	  ## -> string
	end

    # To be added
	def physicalPath
	  ## -> string
	  return( serverVariables( 'DOCUMENT_ROOT' ) . serverVariables( "SCRIPT_NAME" ) )
	end

    alias :rawUrl :currentExecutionFilePath

    alias :requestType :method

    # TODO
	def url
	  ## -> System.Uri
	end

    # TODO
	def urlReferrer
	  ## -> System.Uri
	end

    # To be added
	def userAgent
	  ## -> string
	  return @oApacheRequest.headers_in['User-Agent']
	end

    # To be added
	def userHostAddress
	  ## -> string
	  return @oApacheRequest.headers_in['Host']
    end
	
    # TODO
	def userHostName
	  ## -> string
	end

    # TODO
	def userLanguages
	  ## -> string[]
	end

    # TODO
    def binaryRead( iCount )
	  ## -> bytes[]
	end

    # TODO
    def mapImageCoordinates( xImageFileName )
	  ## -> int[]
	end
	
    # TODO
	def mapPath( xVirtualPath, xBaseVirtualDir = nil, bAllowCrossAppMapping = nil )
	  ## -> string
	end

    # TODO
	def saveAs( xFilename, bIncludeHeaders )
	  ## -> void
	end

    # TODO
	def validateInput( )
	  ## -> void
	end

    private 
## -- ##    def parseCookie( value )
## -- ##      values = Hash::new
## -- ##
## -- ##      parts = value.split( /[\&\;]/ )
## -- ##      parts.each { |v|
## -- ##        key, value = v.split( /\=/, 2 ).map { |x|
## -- ##          Asp::Common::url_decode( x )
## -- ##        }
## -- ##        values[key] = value
## -- ##      }
## -- ##
## -- ##      return values
## -- ##    end

    def getCookies
      cookiesStr = @oApacheRequest.headers_in['Cookie']
      cookies = Hash::new

      if cookiesStr.nil? == false
        parts = cookiesStr.split( /;\s*/ )
        parts.each { |v|
          name, value = v.split( /\=/, 2 )
          name = Asp::Common::url_decode( name )
          #next if name == "session-id"
          next if name == Asp::Common::sessionID
          next if cookies[name].nil? == false
          if /\=/ =~ value
## -- ##            cookies[name] = parseCookie( value )
            cookies[name] = Asp::Common::parseCookie( value )
          else
            cookies[name] = Asp::Common::url_decode( value )
          end
        }
      end

      return cookies
    end
    
    def initialize( r, oAspError, aspHandler )
      @oApacheRequest = r
      @oAspError = oAspError
      @oAspHandler = aspHandler
      
      @param = Asp::Params::new( @oApacheRequest )
      @params = @param.params
      @cookies = getCookies
    end
  
  end
end
